@@ -270,8 +270,8 @@ void closechansess(struct Channel *channel) {
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].chansess == chansess) {
 			assert(ses.childpids[i].pid > 0);
-			TRACE(("closing pid %d\n", ses.childpids[i].pid));
-			TRACE(("exited = %d\n", chansess->exited));
+			TRACE(("closing pid %d", ses.childpids[i].pid));
+			TRACE(("exited = %d", chansess->exited));
 			ses.childpids[i].pid = -1;
 			ses.childpids[i].chansess = NULL;
 		}
@@ -300,15 +300,13 @@ void chansessionrequest(struct Channel *channel) {
 	wantreply = buf_getbyte(ses.payload);
 
 	if (typelen > MAX_NAME_LEN) {
-		send_msg_channel_failure(channel);
-		m_free(type);
 		TRACE(("leave chansessionrequest: type too long")); /* XXX send error?*/
-		return;
+		goto out;
 	}
 
 	chansess = (struct ChanSess*)channel->typedata;
 	assert(chansess != NULL);
-	TRACE(("type is %s\n", type));
+	TRACE(("type is %s", type));
 
 	if (strcmp(type, "window-change") == 0) {
 		ret = sessionwinchange(chansess);
@@ -334,6 +332,8 @@ void chansessionrequest(struct Channel *channel) {
 		/* etc, todo "env", "subsystem" */
 	}
 
+out:
+
 	if (wantreply) {
 		if (ret == 0) {
 			send_msg_channel_success(channel);
@@ -448,16 +448,26 @@ static int sessionpty(struct ChanSess * chansess) {
 		unsigned char opcode;
 		unsigned int value;
 		const struct TermCode * termcode;
+		unsigned int len;
+
+		len = buf_getint(ses.payload);
+		if (len != ses.payload->len - ses.payload->pos) {
+			dropbear_exit("bad term mode string");
+		}
 
 		while (((opcode = buf_getbyte(ses.payload)) != 0x00) &&
 				opcode <= 159) {
+
+			/* must be before checking type, so that value is consumed even if
+			 * we don't use it */
+			value = buf_getint(ses.payload);
+
 			/* handle types of code */
 			if (opcode > MAX_TERMCODE) {
 				continue;
 			}
 			termcode = &termcodes[(unsigned int)opcode];
 			
-			value = buf_getint(ses.payload);
 
 			switch (termcode->type) {
 
@@ -651,8 +661,12 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	TRACE(("enter ptycommand"));
 
-	/* we already have a pty allocated */
-	assert(chansess->master != -1 && chansess->tty != NULL);
+	/* we need to have a pty allocated */
+	if (chansess->master == -1 || chansess->tty == NULL) {
+		dropbear_log(LOG_WARNING, "no pty was allocated, couldn't execute");
+		return DROPBEAR_FAILURE;
+	}
+	
 	pid = fork();
 	if (pid < 0)
 		return DROPBEAR_FAILURE;
