@@ -27,6 +27,7 @@ static const char *err_2_str[] =
    "A private PK key is required.",
 
    "Invalid argument provided.",
+   "File Not Found",
 
    "Invalid PK type.",
    "Invalid PK system.",
@@ -34,7 +35,16 @@ static const char *err_2_str[] =
    "Key not found in keyring.",
    "Invalid sized parameter.",
 
-   "Invalid size for prime."
+   "Invalid size for prime.",
+
+};
+
+static const struct {
+    int mpi_code, ltc_code;
+} mpi_to_ltc_codes[] = {
+   { MP_OKAY ,  CRYPT_OK},
+   { MP_MEM  ,  CRYPT_MEM},
+   { MP_VAL  ,  CRYPT_INVALID_ARG},
 };
 
 const char *error_to_string(int err)
@@ -46,3 +56,17 @@ const char *error_to_string(int err)
    }   
 }
 
+/* convert a MPI error to a LTC error (Possibly the most powerful function ever!  Oh wait... no) */
+int mpi_to_ltc_error(int err)
+{
+   int x;
+
+   for (x = 0; x < (int)(sizeof(mpi_to_ltc_codes)/sizeof(mpi_to_ltc_codes[0])); x++) {
+       if (err == mpi_to_ltc_codes[x].mpi_code) { 
+          return mpi_to_ltc_codes[x].ltc_code;
+       }
+   }
+   return CRYPT_ERROR;
+}
+
+
