@@ -27,28 +27,36 @@ mp_radix_size (mp_int * a, int radix)
     return mp_count_bits (a) + (a->sign == MP_NEG ? 1 : 0) + 1;
   }
 
+  /* make sure the radix is in range */
   if (radix < 2 || radix > 64) {
     return 0;
   }
 
+  /* init a copy of the input */
   if ((res = mp_init_copy (&t, a)) != MP_OKAY) {
     return 0;
   }
 
+  /* digs is the digit count */
   digs = 0;
+
+  /* if it's negative add one for the sign */
   if (t.sign == MP_NEG) {
     ++digs;
     t.sign = MP_ZPOS;
   }
 
+  /* fetch out all of the digits */
   while (mp_iszero (&t) == 0) {
     if ((res = mp_div_d (&t, (mp_digit) radix, &t, &d)) != MP_OKAY) {
       mp_clear (&t);
-      return 0;
+      return res;
     }
     ++digs;
   }
   mp_clear (&t);
+
+  /* return digs + 1, the 1 is for the NULL byte that would be required. */
   return digs + 1;
 }
 
